import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import calendarize from 'calendarize';
import { __rest } from 'tslib';
import { Component, EventEmitter, Input, Output, ElementRef, HostListener, Pipe, NgModule } from '@angular/core';

const dayjs = require("dayjs");
class CalendarComponent {
    constructor() {
        this.singleCalendar = false;
        this.dateChanged = new EventEmitter();
        this.monthChanged = new EventEmitter();
        this.yearChanged = new EventEmitter();
    }
    get monthText() {
        return dayjs([+this.year, +this.month]).format("MMM");
    }
    ngOnChanges(changes) {
        this.createCalendarGridData();
    }
    createCalendarGridData() {
        let year = null;
        let month = null;
        const thisMonthStartDate = dayjs([this.year, this.month]).startOf("month");
        const previousMonthStartDate = thisMonthStartDate
            .subtract(1, "month")
            .startOf("month");
        const nextMonthStartDate = thisMonthStartDate
            .add(1, "month")
            .startOf("month");
        year = previousMonthStartDate.get("year");
        month = previousMonthStartDate.get("month");
        const previousMonthLastWeek = calendarize(previousMonthStartDate.toDate())
            .pop()
            .filter(Boolean)
            .map((day) => {
            return dayjs([year, month, day, 0, 0, 0, 0]);
        });
        year = thisMonthStartDate.get("year");
        month = thisMonthStartDate.get("month");
        const thisMonthweekList = calendarize(thisMonthStartDate.toDate()).map((week) => {
            return week.filter(Boolean).map((day) => {
                if (day === 0) {
                    return null;
                }
                return dayjs([year, month, day, 0, 0, 0, 0]);
            });
        });
        year = nextMonthStartDate.get("year");
        month = nextMonthStartDate.get("month");
        const nextMonthFirstWeek = calendarize(nextMonthStartDate.toDate())
            .shift()
            .filter(Boolean)
            .map((day) => {
            return dayjs([year, month, day, 0, 0, 0, 0]);
        });
        if (thisMonthweekList[0].length < 7) {
            thisMonthweekList[0] = previousMonthLastWeek.concat(thisMonthweekList[0]);
        }
        if (thisMonthweekList.slice(-1)[0].length < 7) {
            thisMonthweekList[thisMonthweekList.length - 1] = thisMonthweekList
                .slice(-1)[0]
                .concat(nextMonthFirstWeek);
        }
        if (thisMonthweekList.length < 6) {
            year = previousMonthStartDate.get("year");
            month = previousMonthStartDate.get("month");
            const previousMonthSecondLastWeek = calendarize(previousMonthStartDate.toDate())
                .slice(-2)[0]
                .filter(Boolean)
                .map((day) => {
                return dayjs([year, month, day, 0, 0, 0, 0]);
            });
            thisMonthweekList.unshift(previousMonthSecondLastWeek);
        }
        this.weekList = thisMonthweekList;
    }
    isDisabled(day) {
        return (day.isBefore(dayjs(this.minDate, this.format)) ||
            day.isAfter(dayjs(this.maxDate, this.format)) ||
            (day.isBefore(dayjs(this.selectedFromDate, this.format)) &&
                this.disableBeforeStart &&
                !this.isLeft));
    }
    isDateAvailable(day) {
        if (day.get("month") !== this.month) {
            return false;
        }
        if (!this.singleCalendar &&
            this.inactiveBeforeStart &&
            day.isBefore(this.selectedFromDate, "date")) {
            return false;
        }
        return true;
    }
    isSelectedDate(day) {
        if (day.get("month") === this.month &&
            day.isSame(this.selectedFromDate, "date")) {
            return true;
        }
        if (!this.singleCalendar &&
            day.get("month") === this.month &&
            day.isSameOrAfter(this.selectedFromDate, "date") &&
            day.isSameOrBefore(this.selectedToDate, "date")) {
            return true;
        }
    }
    getFormattedDate(day) {
        return day.format(this.format);
    }
    dateSelected(day) {
        this.dateChanged.emit({
            day: day,
            isLeft: this.isLeft,
        });
    }
    monthSelected(value) {
        this.monthChanged.emit({
            value: value,
            isLeft: this.isLeft,
        });
    }
    yearSelected(value) {
        this.yearChanged.emit({
            value: value,
            isLeft: this.isLeft,
        });
    }
}
CalendarComponent.decorators = [
    { type: Component, args: [{
                selector: "calendar",
                template: "<div class=\"drp-flex drp-space-around\">\n  <button class=\"drp-btn flat\" (click)=\"yearSelected(-1)\" title=\"Previous year\" aria-label=\"Previous year\">\n    <doubleChevronLeft></doubleChevronLeft>\n  </button>\n  <button class=\"drp-btn flat\" (click)=\"monthSelected(-1)\" title=\"Previous month\" aria-label=\"Previous month\">\n    <chevronLeft></chevronLeft>\n  </button>\n  <label class=\"mb-0\"> {{monthText}} {{year}} </label>\n  <button class=\"drp-btn flat next\" (click)=\"monthSelected(1)\" title=\"Next month\" aria-label=\"Next month\">\n    <chevronLeft class=\"right\"></chevronLeft>\n  </button>\n  <button class=\"drp-btn flat next\" (click)=\"yearSelected(1)\" title=\"Next Year\" aria-label=\"Next Year\">\n    <doubleChevronLeft class=\"right\"></doubleChevronLeft>\n  </button>\n</div>\n<div class=\"calendar-grid\">\n  <table>\n    <thead>\n      <th>Su</th>\n      <th>Mo</th>\n      <th>Tu</th>\n      <th>We</th>\n      <th>Th</th>\n      <th>Fr</th>\n      <th>Sa</th>\n    </thead>\n    <tbody>\n      <tr *ngFor=\"let week of weekList; let i = index\">\n        <td *ngFor=\"let day of weekList[i]\" (click)=\"dateSelected(day)\" class=\"drp-date\"\n          [ngClass]=\"{'off':!isDateAvailable(day),'active':isSelectedDate(day),'disabled':isDisabled(day)}\">\n          <button class=\"drp-btn flat\" [attr.aria-label]=\"day | formatDate: format\">{{ day.format('D') }}</button>\n        </td>\n      </tr>\n    </tbody>\n  </table>\n</div>\n<time-picker *ngIf=\"timePicker\" [options]=\"timePicker\" [selectedFromDate]=\"selectedFromDate\"\n  [selectedToDate]=\"selectedToDate\" [minDate]=\"minDate\" [maxDate]=\"maxDate\" [format]=\"format\" [isLeft]=\"isLeft\"\n  (timeChanged)=\"dateSelected($event)\"></time-picker>\n"
            }] }
];
CalendarComponent.propDecorators = {
    month: [{ type: Input }],
    year: [{ type: Input }],
    selectedFromDate: [{ type: Input }],
    selectedToDate: [{ type: Input }],
    isLeft: [{ type: Input }],
    format: [{ type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    inactiveBeforeStart: [{ type: Input }],
    disableBeforeStart: [{ type: Input }],
    timePicker: [{ type: Input }],
    singleCalendar: [{ type: Input }],
    dateChanged: [{ type: Output }],
    monthChanged: [{ type: Output }],
    yearChanged: [{ type: Output }]
};

const dayjs$1 = require("dayjs");
const defaults = {
    ranges: [
        {
            name: "Today",
            value: {
                start: dayjs$1(),
                end: dayjs$1(),
            },
        },
        {
            name: "Yesterday",
            value: {
                start: dayjs$1().subtract(1, "days"),
                end: dayjs$1().subtract(1, "days"),
            },
        },
        {
            name: "last 7 days",
            value: {
                start: dayjs$1().subtract(6, "days"),
                end: dayjs$1(),
            },
        },
        {
            name: "This month",
            value: {
                start: dayjs$1().startOf("month"),
                end: dayjs$1().endOf("month"),
            },
        },
        {
            name: "Last Month",
            value: {
                start: dayjs$1().subtract(1, "month").startOf("month"),
                end: dayjs$1().subtract(1, "month").endOf("month"),
            },
        },
    ],
};

class DefinedDateRange {
}
class Timepicker {
    constructor() {
        this.minuteInterval = 1;
        this.twentyFourHourFormat = true;
    }
}
class YearsGrid {
}
class Options {
    constructor() {
        this.startDate = null;
        this.endDate = null;
        this.minDate = null;
        this.maxDate = null;
        this.format = "YYYY-MM-DD";
        this.inactiveBeforeStart = false;
        this.autoApply = false;
        this.singleCalendar = false;
        this.noDefaultRangeSelected = false;
        this.showRanges = false;
        this.position = "left";
        this.disabled = false;
        this.timePicker = null;
        this.disableBeforeStart = false;
        this.alwaysOpen = false;
        this.theme = "light";
        this.required = false;
        this.DOB = false;
    }
}

const dayjs$2 = require("dayjs");
class DaterangepickerComponent {
    constructor(elem) {
        this.elem = elem;
        this.rangeSelected = new EventEmitter();
        this.range = "";
        this.enableApplyButton = false;
        this.areOldDatesStored = false;
    }
    handleOutsideClick(event) {
        if (!this.derivedOptions.disabled) {
            let current = event.target;
            let host = this.elem.nativeElement;
            if (host.compareDocumentPosition) {
                if (host.compareDocumentPosition(current) &
                    window.Node.DOCUMENT_POSITION_CONTAINED_BY) {
                    this.storeOldDates();
                    return this.toggleCalendars(true);
                }
            }
            else if (host.contains) {
                if (host.contains(current)) {
                    this.storeOldDates();
                    return this.toggleCalendars(true);
                }
            }
            else {
                do {
                    if (current === host) {
                        this.storeOldDates();
                        return this.toggleCalendars(true);
                    }
                    current = current.parentNode;
                } while (current);
            }
            if (this.showCalendars) {
                if (!this.derivedOptions.autoApply) {
                    this.restoreOldDates();
                }
                this.toggleCalendars(false);
            }
        }
    }
    ngDoCheck() {
        this.deriveOptions(true);
    }
    ngOnInit() {
        this.deriveOptions();
        this.validateMinMaxDates();
        this.setFromDate(this.derivedOptions.startDate);
        this.setToDate(this.derivedOptions.endDate);
        this.updateCalendar();
    }
    deriveOptions(isUpdate = false) {
        if (isUpdate) {
            const _a = this.options, restOptions = __rest(_a, ["startDate", "endDate", "minDate", "maxDate"]);
            this.derivedOptions = Object.assign({}, new Options(), this.derivedOptions, restOptions);
        }
        else {
            this.derivedOptions = Object.assign({}, new Options(), this.options);
        }
        if (this.derivedOptions.singleCalendar) {
            this.derivedOptions.autoApply = true;
            this.derivedOptions.endDate = null;
        }
        if (this.derivedOptions.timePicker) {
            this.derivedOptions.autoApply = false;
        }
        if (!this.derivedOptions.displayFormat) {
            this.derivedOptions.displayFormat = this.derivedOptions.format;
        }
        if (this.derivedOptions.showRanges &&
            !this.derivedOptions.preDefinedRanges) {
            this.derivedOptions.preDefinedRanges = defaults.ranges;
        }
        if (window.innerWidth > 600) {
            if (this.derivedOptions.position === "left") {
                const spaceToRight = window.innerWidth -
                    this.elem.nativeElement.getBoundingClientRect().left;
                if (spaceToRight < 500) {
                    console.log("object");
                    this.derivedOptions.position = "right";
                }
            }
            else if (this.derivedOptions.position === "right") {
                if (this.elem.nativeElement.getBoundingClientRect().right < 500) {
                    this.derivedOptions.position = "left";
                }
            }
        }
    }
    setFromDate(value) {
        if (this.derivedOptions.noDefaultRangeSelected && !value) {
            this.fromDate = "";
            this.tempFromDate = this.getActualFromDate(value);
        }
        else {
            this.fromDate = this.getActualFromDate(value);
        }
    }
    getActualFromDate(value) {
        let temp;
        if ((temp = this.getValidateDayjs(value))) {
            return this.getValidateFromDate(temp);
        }
        return this.getValidateFromDate(dayjs$2());
    }
    setToDate(value) {
        if (this.derivedOptions.noDefaultRangeSelected && !value) {
            this.toDate = "";
            this.tempToDate = this.getActualToDate(value);
        }
        else {
            this.toDate = this.getActualToDate(value);
        }
    }
    getActualToDate(value) {
        let temp;
        if ((temp = this.getValidateDayjs(value))) {
            return this.getValidateToDate(temp);
        }
        return this.getValidateToDate(dayjs$2());
    }
    validateMinMaxDates() {
        if (this.derivedOptions) {
            if (this.derivedOptions.minDate && !this.derivedOptions.maxDate) {
                this.derivedOptions.minDate = this.getDayjs(this.derivedOptions.minDate);
            }
            if (!this.derivedOptions.minDate && this.derivedOptions.maxDate) {
                this.derivedOptions.maxDate = this.getDayjs(this.derivedOptions.maxDate);
            }
            if (this.derivedOptions.minDate && this.derivedOptions.maxDate) {
                this.derivedOptions.minDate = this.getDayjs(this.derivedOptions.minDate);
                this.derivedOptions.maxDate = this.getDayjs(this.derivedOptions.maxDate);
                if (this.derivedOptions.maxDate.isBefore(this.derivedOptions.minDate, "date")) {
                    this.derivedOptions.minDate = "";
                    this.derivedOptions.maxDate = "";
                    console.warn("supplied minDate is after maxDate. Discarding options for minDate and maxDate");
                }
            }
            if (this.derivedOptions.minDate &&
                this.derivedOptions.minDate.format("HH:mm") === "00:00") {
                this.derivedOptions.minDate.set({
                    hour: 0,
                    minutes: 0,
                    seconds: 0,
                });
            }
            if (this.derivedOptions.maxDate &&
                this.derivedOptions.maxDate.format("HH:mm") === "00:00") {
                this.derivedOptions.maxDate.set({
                    hour: 23,
                    minutes: 59,
                    seconds: 59,
                });
            }
        }
    }
    getValidateFromDate(value) {
        if (!this.derivedOptions.timePicker) {
            if (this.derivedOptions.minDate &&
                this.derivedOptions.maxDate &&
                value.isSameOrAfter(this.derivedOptions.minDate, "date") &&
                value.isSameOrBefore(this.derivedOptions.maxDate, "date")) {
                return value;
            }
            if (this.derivedOptions.minDate &&
                !this.derivedOptions.maxDate &&
                value.isAfter(this.derivedOptions.minDate, "date")) {
                return value;
            }
            if (this.derivedOptions.minDate) {
                return this.derivedOptions.minDate.clone();
            }
            return dayjs$2();
        }
        else {
            if (this.derivedOptions.minDate &&
                this.derivedOptions.maxDate &&
                value.isSameOrAfter(this.derivedOptions.minDate, this.derivedOptions.format) &&
                value.isSameOrBefore(this.derivedOptions.maxDate, this.derivedOptions.format)) {
                return value;
            }
            if (this.derivedOptions.minDate &&
                !this.derivedOptions.maxDate &&
                value.isAfter(this.derivedOptions.minDate, this.derivedOptions.format)) {
                return value;
            }
            if (this.derivedOptions.minDate) {
                return this.derivedOptions.minDate.clone();
            }
            return dayjs$2();
        }
    }
    getValidateToDate(value) {
        if (!this.derivedOptions.timePicker) {
            if ((this.derivedOptions.maxDate &&
                value.isSameOrAfter(this.fromDate, "date"),
                value.isSameOrBefore(this.derivedOptions.maxDate, "date"))) {
                return value;
            }
            if (this.derivedOptions.maxDate) {
                return this.derivedOptions.maxDate.clone();
            }
            return dayjs$2();
        }
        else {
            if ((this.derivedOptions.maxDate &&
                value.isSameOrAfter(this.fromDate, this.derivedOptions.format),
                value.isSameOrBefore(this.derivedOptions.maxDate, this.derivedOptions.format))) {
                return value;
            }
            if (this.derivedOptions.maxDate) {
                return this.derivedOptions.maxDate.clone();
            }
            return dayjs$2();
        }
    }
    getDayjs(value) {
        return dayjs$2(value, this.derivedOptions.format);
    }
    getValidateDayjs(value) {
        let dayjsValue = null;
        if (dayjs$2(value, this.derivedOptions.format, true).isValid()) {
            dayjsValue = dayjs$2(value, this.derivedOptions.format, true);
        }
        return dayjsValue;
    }
    formatFromDate(event) {
        if (event.target.value !== this.fromDate.format(this.derivedOptions.format)) {
            this.dateChanged({
                day: event.target.value ? this.getDayjs(event.target.value) : dayjs$2(),
                isLeft: true,
            });
        }
    }
    formatToDate(event) {
        if (event.target.value !== this.toDate.format(this.derivedOptions.format)) {
            this.dateChanged({
                day: event.target.value ? this.getDayjs(event.target.value) : dayjs$2(),
                isLeft: false,
            });
        }
    }
    dateChanged(data) {
        let value = data.day;
        let isLeft = data.isLeft;
        if (isLeft) {
            if (!this.derivedOptions.timePicker) {
                value = value.hour(0);
                value = value.minute(0);
                value = value.second(0);
            }
            this.fromDate = value;
            if (!this.derivedOptions.timePicker) {
                if (value.isAfter(this.toDate, "date")) {
                    this.toDate = this.fromDate.clone();
                }
            }
            else {
                if (value.isAfter(this.toDate, this.derivedOptions.format)) {
                    this.toDate = this.fromDate.clone();
                }
            }
        }
        else {
            if (!this.derivedOptions.timePicker) {
                value = value.hour(23);
                value = value.minute(59);
                value = value.second(59);
            }
            this.toDate = value;
            this.toYear = this.toDate.get("year");
            if (!this.derivedOptions.timePicker) {
                if (value.isBefore(this.fromDate, "date")) {
                    this.fromDate = this.toDate.clone();
                }
            }
            else {
                if (value.isBefore(this.fromDate, this.derivedOptions.format)) {
                    this.fromDate = this.toDate.clone();
                }
            }
        }
        if (this.derivedOptions.autoApply) {
            if (this.derivedOptions.singleCalendar || !isLeft) {
                this.toggleCalendars(false);
                this.setRange();
                this.emitRangeSelected();
            }
        }
        else if (!this.derivedOptions.singleCalendar && !isLeft) {
            this.enableApplyButton = true;
        }
        else if (this.derivedOptions.singleCalendar) {
            this.enableApplyButton = true;
        }
        this.fromMonth = this.fromDate
            ? this.fromDate.get("month")
            : this.fromMonth;
        this.toMonth = this.toDate ? this.toDate.get("month") : this.toMonth;
        this.fromYear = this.fromDate.get("year");
        if (!this.toDate && this.fromDate) {
            this.toYear = this.fromYear;
            this.toMonth = this.fromMonth;
        }
        if (this.toYear < this.fromYear) {
            this.toYear = this.fromYear;
        }
    }
    monthChanged(data) {
        let temp;
        if (data.isLeft) {
            temp = dayjs$2([this.fromYear, this.fromMonth]).add(data.value, "months");
            this.fromMonth = temp.get("month");
            this.fromYear = temp.get("year");
        }
        else {
            temp = dayjs$2([this.toYear, this.toMonth]).add(data.value, "months");
            this.toMonth = temp.get("month");
            this.toYear = temp.get("year");
        }
    }
    yearChanged(data) {
        let temp;
        if (data.isLeft) {
            temp = dayjs$2([this.fromYear, this.fromMonth]).add(data.value, "year");
            this.fromMonth = temp.get("month");
            this.fromYear = temp.get("year");
        }
        else {
            temp = dayjs$2([this.toYear, this.toMonth]).add(data.value, "year");
            this.toMonth = temp.get("month");
            this.toYear = temp.get("year");
        }
    }
    emitRangeSelected() {
        let data = {};
        if (this.derivedOptions.singleCalendar) {
            data = {
                start: this.getDayjs(this.fromDate),
            };
        }
        else {
            data = {
                start: this.getDayjs(this.fromDate),
                end: this.getDayjs(this.toDate),
            };
        }
        this.rangeSelected.emit(data);
    }
    storeOldDates() {
        if (!this.areOldDatesStored) {
            this.oldFromDate = this.fromDate;
            this.oldToDate = this.toDate;
            this.areOldDatesStored = true;
        }
    }
    restoreOldDates() {
        this.fromDate = this.oldFromDate;
        this.toDate = this.oldToDate;
    }
    setRange() {
        if (this.derivedOptions.singleCalendar && this.fromDate) {
            this.range = this.fromDate.format(this.derivedOptions.displayFormat);
        }
        else if (this.fromDate && this.toDate) {
            this.range =
                this.fromDate.format(this.derivedOptions.displayFormat) +
                    " - " +
                    this.toDate.format(this.derivedOptions.displayFormat);
        }
        else {
            this.range = "";
        }
    }
    apply() {
        this.toggleCalendars(false);
        this.setRange();
        this.emitRangeSelected();
    }
    cancel() {
        this.restoreOldDates();
        this.toggleCalendars(false);
    }
    clear() {
        this.fromDate = this.toDate = "";
        this.apply();
        this.enableApplyButton = false;
        this.fromYear = this.toYear = dayjs$2().get("year");
        this.fromMonth = this.toMonth = dayjs$2().get("month");
    }
    applyPredefinedRange(data) {
        this.setFromDate(data.value.start);
        this.setToDate(data.value.end);
        this.toggleCalendars(false);
        this.emitRangeSelected();
    }
    toggleCalendars(value) {
        this.showCalendars = value;
        if (!value) {
            this.areOldDatesStored = false;
            this.updateCalendar();
        }
    }
    updateCalendar() {
        let fromDate = dayjs$2(this.fromDate).isValid()
            ? this.fromDate
            : this.tempFromDate;
        let toDate = dayjs$2(this.toDate).isValid() ? this.toDate : this.tempToDate;
        let tDate = dayjs$2(fromDate, this.derivedOptions.format);
        this.fromMonth = tDate.get("month");
        this.fromYear = tDate.get("year");
        tDate = dayjs$2(toDate, this.derivedOptions.format);
        this.toMonth = tDate.get("month");
        this.toYear = tDate.get("year");
        this.setRange();
    }
    getAriaLabel() {
        if (this.fromDate && this.toDate) {
            return (this.fromDate.format(this.derivedOptions.displayFormat) +
                " to " +
                this.toDate.format(this.derivedOptions.displayFormat));
        }
        return "Please select a date range";
    }
}
DaterangepickerComponent.decorators = [
    { type: Component, args: [{
                selector: "daterangepicker",
                template: "<div class=\"drp-wrapper {{derivedOptions.theme}}\">\n  <input class=\"{{class}} drp-input\" type=\"text\" [ngModel]=\"range\" [disabled]=\"derivedOptions.disabled\"\n    [attr.aria-label]=\"getAriaLabel()\" [class.hidden]=\"derivedOptions.alwaysOpen\"\n    [attr.required]=\"derivedOptions.required\" />\n  <div class=\"drp-flyout {{derivedOptions.position}}\" [ngClass]=\"{\n          hidden: !(showCalendars || derivedOptions.alwaysOpen),\n          singledatepicker: derivedOptions.singleCalendar,\n          tooltipChevron: !derivedOptions.alwaysOpen,\n          alwaysOpen: derivedOptions.alwaysOpen\n        }\">\n    <div class=\"drp-calendars-wrapper\">\n      <div class=\"drp-calendar-container\">\n        <input class=\"drp-calendar-input\" [ngModel]=\"fromDate | formatDate: derivedOptions.displayFormat\"\n          (blur)=\"formatFromDate($event)\" [class.hidden]=\"derivedOptions.singleCalendar\" readonly type=\"text\"\n          name=\"daterangepicker_start\" />\n        <calendar class=\"calender\" [isLeft]=\"true\" [month]=\"fromMonth\" [year]=\"fromYear\"\n          (monthChanged)=\"monthChanged($event)\" (yearChanged)=\"yearChanged($event)\" (dateChanged)=\"dateChanged($event)\"\n          [format]=\"format\" [selectedFromDate]=\"fromDate\" [selectedToDate]=\"toDate\" [minDate]=\"derivedOptions.minDate\"\n          [maxDate]=\"derivedOptions.maxDate\" [inactiveBeforeStart]=\"derivedOptions.inactiveBeforeStart\"\n          [disableBeforeStart]=\"derivedOptions.disableBeforeStart\" [timePicker]=\"derivedOptions.timePicker\"\n          [singleCalendar]=\"derivedOptions.singleCalendar\"></calendar>\n      </div>\n      <div class=\"drp-calendar-container\" *ngIf=\"!derivedOptions.singleCalendar\">\n        <input class=\"drp-calendar-input\" [ngModel]=\"toDate | formatDate: derivedOptions.displayFormat\"\n          (blur)=\"formatToDate($event)\" readonly name=\"daterangepicker_end\" />\n        <calendar class=\"calender\" [month]=\"toMonth\" [year]=\"toYear\" [format]=\"format\"\n          (dateChanged)=\"dateChanged($event)\" (monthChanged)=\"monthChanged($event)\" (yearChanged)=\"yearChanged($event)\"\n          [selectedFromDate]=\"fromDate\" [selectedToDate]=\"toDate\" [minDate]=\"derivedOptions.minDate\"\n          [maxDate]=\"derivedOptions.maxDate\" [inactiveBeforeStart]=\"derivedOptions.inactiveBeforeStart\"\n          [disableBeforeStart]=\"derivedOptions.disableBeforeStart\" [timePicker]=\"derivedOptions.timePicker\"></calendar>\n      </div>\n    </div>\n    <div class=\"my-0\">\n      <div class=\"drp-flex\"><button [class.hidden]=\"derivedOptions.autoApply\" class=\"drp-btn apply\"\n          [disabled]=\"!enableApplyButton\" (click)=\"apply()\" type=\"button\">\n          Apply\n        </button>\n        <button [class.hidden]=\"derivedOptions.autoApply\" class=\"drp-btn cancel\" (click)=\"cancel()\" type=\"button\">\n          Cancel\n        </button>\n        <button [disabled]=\"!this.range\" class=\"drp-btn flat link\" (click)=\"clear()\" type=\"button\">\n          Clear\n        </button>\n      </div>\n      <div class=\"drp-flex range-controls\" *ngIf=\"derivedOptions.showRanges && !derivedOptions.singleCalendar\">\n        <button *ngFor=\"let range of derivedOptions.preDefinedRanges\" class=\"drp-btn outline\"\n          (click)=\"applyPredefinedRange(range)\" type=\"button\">\n          {{ range.name }}\n        </button>\n      </div>\n    </div>\n  </div>\n</div>\n"
            }] }
];
DaterangepickerComponent.ctorParameters = () => [
    { type: ElementRef }
];
DaterangepickerComponent.propDecorators = {
    options: [{ type: Input }],
    class: [{ type: Input }],
    rangeSelected: [{ type: Output }],
    handleOutsideClick: [{ type: HostListener, args: ["document:mousedown", ["$event"],] }, { type: HostListener, args: ["document:mouseup", ["$event"],] }, { type: HostListener, args: ["document:keyup", ["$event"],] }]
};

class FormatDatePipe {
    transform(value, format) {
        if (value)
            return value.format(format);
        return "";
    }
}
FormatDatePipe.decorators = [
    { type: Pipe, args: [{ name: 'formatDate' },] }
];

class chevronLeft {
}
chevronLeft.decorators = [
    { type: Component, args: [{
                selector: "chevronLeft",
                template: `
    <svg
      xmlns="http://www.w3.org/2000/svg"
      xmlns:xlink="http://www.w3.org/1999/xlink"
      version="1.1"
      id="Capa_1"
      x="0px"
      y="0px"
      width="16px"
      height="16px"
      viewBox="0 0 444.531 444.531"
      style="enable-background:new 0 0 444.531 444.531;"
      xml:space="preserve"
    >
      <g>
        <path
          d="M213.13,222.409L351.88,83.653c7.05-7.043,10.567-15.657,10.567-25.841c0-10.183-3.518-18.793-10.567-25.835   l-21.409-21.416C323.432,3.521,314.817,0,304.637,0s-18.791,3.521-25.841,10.561L92.649,196.425   c-7.044,7.043-10.566,15.656-10.566,25.841s3.521,18.791,10.566,25.837l186.146,185.864c7.05,7.043,15.66,10.564,25.841,10.564   s18.795-3.521,25.834-10.564l21.409-21.412c7.05-7.039,10.567-15.604,10.567-25.697c0-10.085-3.518-18.746-10.567-25.978   L213.13,222.409z"
        />
      </g>
    </svg>
  `
            }] }
];
chevronLeft.propDecorators = {
    class: [{ type: Input }]
};

class DoubleChevronLeft {
}
DoubleChevronLeft.decorators = [
    { type: Component, args: [{
                selector: "doubleChevronLeft",
                template: `
    <svg
      xmlns="http://www.w3.org/2000/svg"
      xmlns:xlink="http://www.w3.org/1999/xlink"
      version="1.1"
      id="Layer_1"
      x="0px"
      y="0px"
      viewBox="0 0 407.436 407.436"
      style="enable-background:new 0 0 407.436 407.436;"
      xml:space="preserve"
      width="16px"
      height="16px"
    >
      <g>
        <polygon
          points="266.452,21.178 245.204,0 42.149,203.718 245.204,407.436 266.452,386.258 84.507,203.718  "
        />
        <polygon
          points="365.286,21.178 344.038,0 140.983,203.718 344.038,407.436 365.286,386.258 183.341,203.718  "
        />
      </g>
    </svg>
  `
            }] }
];
DoubleChevronLeft.propDecorators = {
    class: [{ type: Input }]
};

const dayjs$3 = require("dayjs");
class TimePickerComponent {
    constructor() {
        this.options = new Timepicker();
        this.timeChanged = new EventEmitter();
    }
    ngOnInit() {
        if (!this.options.minuteInterval ||
            this.options.minuteInterval % 60 === 0) {
            this.options.minuteInterval = 1;
        }
        this.options.minuteInterval = this.options.minuteInterval % 60;
    }
    ngOnChanges(changes) {
        this.selectedFromDate = changes["selectedFromDate"]
            ? dayjs$3(changes["selectedFromDate"].currentValue, this.format)
            : this["selectedFromDate"];
        this.selectedToDate = changes["selectedToDate"]
            ? dayjs$3(changes["selectedToDate"].currentValue, this.format)
            : this["selectedToDate"];
        this.maxDate = changes["maxDate"]
            ? dayjs$3(changes["maxDate"].currentValue, this.format)
            : this["maxDate"];
        this.minDate = changes["minDate"]
            ? dayjs$3(changes["minDate"].currentValue, this.format)
            : this["minDate"];
    }
    getCurrentHour() {
        let currentHour = this.isLeft
            ? this.selectedFromDate.get("hour")
            : this.selectedToDate.get("hour");
        return isNaN(currentHour)
            ? "&mdash;"
            : currentHour > 9
                ? currentHour
                : "0" + currentHour;
    }
    getCurrentMinute() {
        let currentMinute = this.isLeft
            ? this.selectedFromDate.get("minute")
            : this.selectedToDate.get("minute");
        return isNaN(currentMinute)
            ? "&mdash;"
            : currentMinute > 9
                ? currentMinute
                : "0" + currentMinute;
    }
    isValidToAddMinute(value) {
        let possibleNewValue;
        let possibleSelectedDate;
        if (this.isLeft) {
            possibleNewValue = this.selectedFromDate.get("minute") + value;
            possibleSelectedDate = this.selectedFromDate
                .clone()
                .add(value, "minutes");
        }
        else {
            possibleNewValue = this.selectedToDate.get("minute") + value;
            possibleSelectedDate = this.selectedToDate.clone().add(value, "minutes");
        }
        let retValue = possibleNewValue < 60 && possibleNewValue >= 0;
        if (this.minDate.isValid()) {
            retValue = retValue && possibleSelectedDate.isSameOrAfter(this.minDate);
        }
        if (this.maxDate.isValid()) {
            retValue = retValue && possibleSelectedDate.isSameOrBefore(this.maxDate);
        }
        return retValue;
    }
    isValidToAddHour(value) {
        let possibleNewValue;
        let possibleSelectedDate;
        if (this.isLeft) {
            possibleNewValue = this.selectedFromDate.get("hour") + value;
            possibleSelectedDate = this.selectedFromDate.clone().add(value, "hour");
        }
        else {
            possibleNewValue = this.selectedToDate.get("hour") + value;
            possibleSelectedDate = this.selectedToDate.clone().add(value, "hour");
        }
        let retValue = possibleNewValue < 24 && possibleNewValue >= 0;
        if (this.minDate.isValid()) {
            retValue = retValue && possibleSelectedDate.isSameOrAfter(this.minDate);
        }
        if (this.maxDate.isValid()) {
            retValue = retValue && possibleSelectedDate.isSameOrBefore(this.maxDate);
        }
        return retValue;
    }
    addHour(value) {
        if (this.isLeft) {
            this.selectedFromDate = this.selectedFromDate.set("hour", (+this.selectedFromDate.get("hour") + value) % 24);
        }
        else {
            this.selectedToDate = this.selectedToDate.set("hour", (+this.selectedToDate.get("hour") + value) % 24);
        }
        this.triggerTimeChanged();
    }
    addMinute(value) {
        if (this.isLeft) {
            this.selectedFromDate = this.selectedFromDate.set("minute", (this.selectedFromDate.get("minute") + value) % 60);
        }
        else {
            this.selectedToDate = this.selectedToDate.set("minute", (this.selectedToDate.get("minute") + value) % 60);
        }
        this.triggerTimeChanged();
    }
    triggerTimeChanged() {
        this.isLeft
            ? this.timeChanged.emit(this.selectedFromDate)
            : this.timeChanged.emit(this.selectedToDate);
    }
    toggleAMPM() { }
}
TimePickerComponent.decorators = [
    { type: Component, args: [{
                selector: "time-picker",
                template: "<div class=\"drp-flex\"\n  [ngClass]=\"{'col-md-offset-4': options.twentyFourHourFormat, 'col-md-offset-3': !options.twentyFourHourFormat}\">\n  <div class=\"drp-flex drp-column\">\n    <button aria-label=\"Increment hour by 1\" class=\"drp-btn flat up\" [ngClass]=\"{'disabled': !isValidToAddHour(1)}\"\n      (click)=\"addHour(1)\">\n      <chevronLeft></chevronLeft>\n    </button>\n    <span><b [innerHTML]=\"getCurrentHour()\"></b></span>\n    <button aria-label=\"Decrement hour by 1\" class=\"drp-btn flat down\" [ngClass]=\"{'disabled': !isValidToAddHour(-1)}\"\n      (click)=\"addHour(-1)\">\n      <chevronLeft></chevronLeft>\n    </button>\n  </div>\n  <sub>\n    <b>h</b>\n  </sub>\n  <div class=\"drp-flex drp-column\">\n    <button aria-label=\"increment minute by one step\" class=\"drp-btn flat up\"\n      [ngClass]=\"{'disabled': !isValidToAddMinute(options.minuteInterval)}\" (click)=\"addMinute(options.minuteInterval)\">\n      <chevronLeft></chevronLeft>\n    </button>\n    <span><b [innerHTML]=\"getCurrentMinute()\" (mouseWheelUp)=\"addMinute(options.minuteInterval)\"></b></span>\n    <button aria-label=\"decrement minute by one step\" class=\"drp-btn flat down\"\n      [ngClass]=\"{'disabled': !isValidToAddMinute(-1 * options.minuteInterval)}\"\n      (click)=\"addMinute(-1 * options.minuteInterval)\">\n      <chevronLeft></chevronLeft>\n    </button>\n  </div>\n  <sub>\n    <b>m</b>\n  </sub>\n  <div class=\"drp-flex drp-flex-column\" *ngIf=\"!options.twentyFourHourFormat\">\n    <button class=\"drp-btn flat up\" (click)=\"toggleAMPM()\">\n      <chevronLeft></chevronLeft>\n    </button>\n    <span><b>AM</b></span>\n    <button class=\"drp-btn flat down\" (click)=\"toggleAMPM()\">\n      <chevronLeft></chevronLeft>\n    </button>\n  </div>\n</div>\n"
            }] }
];
TimePickerComponent.propDecorators = {
    options: [{ type: Input }],
    selectedFromDate: [{ type: Input }],
    selectedToDate: [{ type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    format: [{ type: Input }],
    isLeft: [{ type: Input }],
    timeChanged: [{ type: Output }]
};

class DatetimerangepickerModule {
}
DatetimerangepickerModule.decorators = [
    { type: NgModule, args: [{
                imports: [FormsModule, CommonModule],
                declarations: [
                    DaterangepickerComponent,
                    CalendarComponent,
                    TimePickerComponent,
                    FormatDatePipe,
                    DoubleChevronLeft,
                    chevronLeft,
                ],
                exports: [DaterangepickerComponent],
            },] }
];

const dayjs$4 = require("dayjs");
const isSameOrAfter = require("dayjs/plugin/isSameOrAfter");
dayjs$4.extend(isSameOrAfter);
const isSameOrBefore = require("dayjs/plugin/isSameOrBefore");
dayjs$4.extend(isSameOrBefore);
const customParser = require("dayjs/plugin/customParseFormat");
dayjs$4.extend(customParser);
const isoWeek = require("dayjs/plugin/isoWeek");
dayjs$4.extend(isoWeek);
const arraySupport = require("dayjs/plugin/arraySupport");
dayjs$4.extend(arraySupport);

export { DatetimerangepickerModule, DefinedDateRange, Timepicker, YearsGrid, Options, CalendarComponent as ɵb, DaterangepickerComponent as ɵa, FormatDatePipe as ɵd, chevronLeft as ɵf, DoubleChevronLeft as ɵe, TimePickerComponent as ɵc };

//# sourceMappingURL=angular-datetimerangepicker.js.map